﻿/**
 * DoGesture
 * @version v.1.01
 * @author 	David Ochmann
 * 
 * @mail	mail@davidochmann.de 	
 * @website www.davidochmann.de
 * 
 * 
 * 
 * v1.01
 * - Bugfix by cerupcat: Added mComplete = false after DoGestureEvent.GESTURE_COMPLETE dispatch in the Logic class.
 *   Prevents DoGesture to trigger the Event after completing a gesture and afterwords clicking the parsed surface. 
 * 
 * v1.02
 * - Removed interfaces to decrease filesize.
 * - Added comments to enhance code structure.
 * - Various code optimizations.
 * - New mObject parameter adds ability to track Object movements.
 * 
 * v1.03
 * - Fixed FlashPlayer 10.2 package name conflict. In existing projects, change the class name GestureEvent to DoGestureEvent.
 */
 
import mx.utils.Delegate;
import mx.events.EventDispatcher;	
import de.davidochmann.utilities.dogesture.move.Move;	 
import de.davidochmann.utilities.dogesture.logic.Logic;	
import de.davidochmann.utilities.dogesture.stack.Stack;	
import de.davidochmann.utilities.dogesture.register.Register;
import de.davidochmann.utilities.dogesture.events.DoGestureEvent;	

class de.davidochmann.utilities.dogesture.DoGesture
{	
	private var mSurface:MovieClip;
	private var mTimeout:Number;
	private var mMoveArray:Array;
	private var mRegister:Register;
	private var mObject:Object;

	private var mStack:Stack;
	private var mLogic:Logic;
	

	public function DoGesture(pSurface:MovieClip,
							  pTimeout:Number,
							  pMoveArray:Array,
							  pStart:Boolean,
							  pObject:Object)
	{
		EventDispatcher.initialize(this);
		
		setSurface(pSurface);
		setTimeout(pTimeout);
		setMoveArray(pMoveArray);
		setObject(pObject);
		
		initStack();
		initLogic();
		initRegister();
		initEventHandler();
		initMoveArray();
		
		if(pStart) start();
	}
	


	/*
	 * PUBLIC FUNCTIONS
	 */	
	
	public function start():Void
	{
		mRegister.start();
		mLogic.start();
	}
	
	public function stop():Void
	{
		mRegister.stop();
		mLogic.stop();
	}
	
	public function active():Boolean
	{
		return mLogic.active();
	}
	
	public function complete():Boolean
	{
		return mLogic.complete();
	}

	public function add(pPositionX:Number, pPositionY:Number):Void
	{			
		var move:Move = new Move(pPositionX, pPositionY);
		mStack.add(move);
	}

	
	
	/*
	 * GETTER FUNCTIONS 
	 */
	
	public function getSurface():MovieClip
	{
		return mSurface;
	}

	public function getTimeout():Number
	{
		return mTimeout;
	}

	public function getMoveArray():Array
	{
		return mMoveArray;
	}

	public function getObject():Object
	{
		return mObject;
	}


	
	/*
	 * SETTER FUNCTIONS
	 */

	public function setSurface(pSurface:MovieClip):Void
	{
		mSurface = pSurface;	
	}

	public function setTimeout(pTimeout:Number):Void
	{					
		mTimeout = pTimeout;
	}

	public function setMoveArray(pMoveArray:Array):Void
	{
		mMoveArray = pMoveArray;
	}

	public function setObject(pObject:Object):Void
	{
		mObject = pObject;
	}

	
	
	/*
	 * PRIVATE FUNCTIONS
	 */
	
	private function initStack():Void
	{
		mStack = new Stack();
	}
	
	private function initLogic():Void
	{
		mLogic = new Logic(mSurface, mStack, mTimeout, mObject);	
	}
	
	private function initRegister():Void
	{
		mRegister = Register.getInstance(mSurface);
	}
	
	private function initEventHandler():Void
	{
		mLogic.addEventListener(DoGestureEvent.MOVE_COMPLETE, Delegate.create(this, logicMoveCompleteHandler));
		mLogic.addEventListener(DoGestureEvent.GESTURE_COMPLETE, Delegate.create(this, logicMoveCompleteHandler));
	}
	
	private function initMoveArray():Void
	{
		if(mMoveArray == null) return;
			
		var obj:Object;
		var move:Move;
			
		for(var i:Number = 0; i < mMoveArray.length; ++i)
		{
			obj  = Object(mMoveArray[i]);
			move = new Move(obj._x, obj._y);
			
			mStack.add(move);
		}			
	}

	private function logicMoveCompleteHandler(e:DoGestureEvent):Void
	{
		dispatchEvent(e);
	}
	
	
	
	/*
	 * EVENT DISPATCHER FUNCTIONS 
	 */
	
	public function addEventListener(type:String, listener:Object):Void{};
	
	public function removeEventListener(type:String, listener:Object):Void{};
	
	private function dispatchEvent(event:Object):Void{};
}

