﻿/**
 * @author dochmann
 */
 
import mx.utils.Delegate;
import mx.events.EventDispatcher;		
import de.davidochmann.utilities.dogesture.move.Move; 
import de.davidochmann.utilities.dogesture.stack.Stack;
import de.davidochmann.utilities.dogesture.register.Register;
import de.davidochmann.utilities.dogesture.events.DoGestureEvent;	 
import de.davidochmann.utilities.dogesture.register.events.RegisterEvent;
	
class de.davidochmann.utilities.dogesture.logic.Logic
{		
	private var mStack:Stack;
	private var mSurface:MovieClip;
	private var mTimeout:Number;
	private var mObject:Object;
	private var mComplete:Boolean;
	private var mRegister:Register;
	private var mTimer:Number;		
	private var mCurrent:Move;
	private var mCompleteX:Boolean;
	private var mCompleteY:Boolean;
	private var mStartX:Number;
	private var mStartY:Number;
	private var mMoveX:Number;
	private var mMoveY:Number;
	private var mCurrentX:Number;
	private var mCurrentY:Number;
	private var mActive:Boolean;
	
	private var mSurfaceMouseMove:Function;
	private var mSurfaceMouseDown:Function;
	private var mSurfaceMouseUp:Function;
	private var mSurfaceEnterFrame:Function;
	
	
	
	public function Logic(pSurface:MovieClip,
						  pStack:Stack,
						  pTimeout:Number,
						  pObject:Object)
	{
		EventDispatcher.initialize(this);
		
		setSurface(pSurface);
		setStack(pStack);
		setTimeout(pTimeout);
		setObject(pObject);
	}
	
		
	
	/*
	 * PUBLIC FUNCTIONS
	 */	
		
	public function start():Void
	{
		initComplete();
		initActive();
		initTimer();
		initRegister();
		initFunctionReference();
		initSurfaceEvents();
	}

	public function stop():Void
	{
		removeRegisterEvents();
		mActive = false;
	}

	public function active():Boolean
	{
		return mActive;
	}

	public function complete():Boolean
	{
		return mComplete;
	}


	
	/*
	 * GETTER FUNCTIONS
	 */
	
	public function getStack():Stack
	{
		return mStack;
	}
	
	public function getTimeout():Number
	{
		return mTimeout;
	}

	public function getSurface():MovieClip
	{
		return mSurface;
	}

	public function getObject():Object
	{
		return mObject;
	}



	/*
	 * SETTER FUNCTIONS
	 */
	
	public function setStack(pStack:Stack):Void
	{
		mStack = pStack;
	}

	public function setTimeout(pTimeout:Number):Void
	{
		mTimeout = pTimeout;
	}

	public function setSurface(pSurface:MovieClip):Void
	{
		mSurface = pSurface;
	}

	public function setObject(pObject:Object):Void
	{
		mObject = pObject;	
	}



	/*
	 * PRIVATE FUNCTIONS
	 */
	
	private function initComplete():Void
	{
		mComplete = false;
	}

	private function initActive():Void
	{
		mActive = true;
	}

	private function initRegister():Void
	{
		mRegister = Register.getInstance();
	}

	private function initFunctionReference():Void
	{
		removeRegisterEvents();
		
		mSurfaceMouseMove  = Delegate.create(this, surfaceMouseMoveHandler);
		mSurfaceMouseDown  = Delegate.create(this, surfaceMouseDownHandler);
		mSurfaceMouseUp    = Delegate.create(this, surfaceMouseUpHandler);
		mSurfaceEnterFrame = Delegate.create(this, surfaceEnterFrameHandler);
	}



	/*
	 * SURFACE EVENTS
	 */

	private function initSurfaceEvents():Void
	{
		if(mObject == undefined)
		{
			mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_DOWN, mSurfaceMouseDown);
			mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_UP  , mSurfaceMouseUp);
		}
		else
		{
			mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_ENTER_FRAME, mSurfaceEnterFrame);
			setStart(mObject._x, mObject._y);
		}
	}
	
	private function surfaceMouseDownHandler():Void
	{
		if(mTimeout != undefined && mTimeout != 0) mTimer = setInterval(this, "timerTimerHandler", mTimeout);
		
		setStart(mSurface._xmouse, mSurface._ymouse);
		mRegister.addEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
	}

	private function surfaceMouseUpHandler():Void
	{
		if(mComplete) dispatchEvent(new DoGestureEvent(DoGestureEvent.GESTURE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
		mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
		
		mActive = false;
		mStack.reset();
	}

	private function setStart(pX:Number, pY:Number):Void
	{
		mActive   = true;
		mComplete = false;
		mCurrent  = mStack.active();
		
		mCurrent.setStartX(pX);
		mCurrent.setStartY(pY);
	}

	private function surfaceMouseMoveHandler():Void
	{
		moveHandler(mSurface._xmouse, mSurface._ymouse);
	}

	private function surfaceEnterFrameHandler():Void
	{
		moveHandler(mObject._x, mObject._y);
	}

	private function moveHandler(pX:Number, pY:Number):Void
	{
		mCompleteX = false;
		mCompleteY = false;
		mStartX	   = mStack.active().getStartX();
		mStartY	   = mStack.active().getStartY();
		mMoveX	   = mStack.active().getPositionX();
		mMoveY	   = mStack.active().getPositionY();
		mCurrentX  = pX;
		mCurrentY  = pY;


		mCompleteX = (mMoveX != 0) ? ((mMoveX > 0) ? (mCurrentX - mStartX > mMoveX) : (mCurrentX - mStartX < mMoveX)) : true;
		mCompleteY = (mMoveY != 0) ? ((mMoveY > 0) ? (mCurrentY - mStartY > mMoveY) : (mCurrentY - mStartY < mMoveY)) : true;
				
		mComplete  = (mCompleteX && mCompleteY);
		
		
		if(mComplete) mStack.next();
	
				
		if(mComplete && !mStack.complete())
		{
			mComplete = false;
			mCurrent  = mStack.active();
			
			mCurrent.setStartX(pX);
			mCurrent.setStartY(pY);
			
			dispatchEvent(new DoGestureEvent(DoGestureEvent.MOVE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
		}
		else if(mComplete && mStack.complete())
		{
			clearInterval(mTimer);
			
			if(mObject == undefined) mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);
			else
			{
				mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_ENTER_FRAME, mSurfaceEnterFrame);					
				dispatchEvent(new DoGestureEvent(DoGestureEvent.GESTURE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
				mActive = false;
				mStack.reset();
			}
			
			dispatchEvent(new DoGestureEvent(DoGestureEvent.MOVE_COMPLETE, this, mComplete, mStack.index(), mStack.list()));
		}	
	}

	private function removeRegisterEvents():Void
	{
		if(mObject == undefined)
		{
			mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_MOVE, mSurfaceMouseMove);	
			mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_DOWN, mSurfaceMouseDown);
			mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_UP  , mSurfaceMouseUp);
		}
		else
			mRegister.removeEventListener(RegisterEvent.MOUSE_GESTURE_ENTER_FRAME, mSurfaceEnterFrame);
	}


	
	/*
	 * TIMER FUNCTIONS
	 */

	private function initTimer():Void
	{
		clearInterval(mTimer);
	}
	
	private function timerTimerHandler():Void
	{	
		clearInterval(mTimer);
		stop();
	}
	
	
	
	/*
	 * EVENT DISPATCHER FUNCTIONS
	 */
	
	public function addEventListener(type:String, listener:Object):Void{};
	
	public function removeEventListener(type:String, listener:Object):Void{};
	
	private function dispatchEvent(event:Object):Void{};
}