﻿package de.davidochmann.utilities.dogesture 
{
	/**
	 * DoGesture
	 * @version v.1.01
	 * @author 	David Ochmann
	 * 
	 * @mail	mail@davidochmann.de 	
	 * @website www.davidochmann.de
	 * 
	 * 
	 * 
	 * v1.01
	 * - Bugfix by cerupcat: Added mComplete = false after DoGestureEvent.GESTURE_COMPLETE dispatch in the Logic class.
	 *   Prevents DoGesture to trigger the Event after completing a gesture and afterwords clicking the parsed surface. 
	 * 
	 * v1.02
	 * - Removed interfaces to decrease filesize.
	 * - Added comments to enhance code structure.
	 * - Various code optimizations.
	 * - New mObject parameter adds ability to track Object movements.
	 * 
	 * v1.03
	 * - Fixed FlashPlayer 10.2 package name conflict. In existing projects, change the class name GestureEvent to DoGestureEvent.
	 */
	 

	import flash.display.DisplayObject;	
	import flash.events.EventDispatcher;	
	import de.davidochmann.utilities.dogesture.move.Move;	 
	import de.davidochmann.utilities.dogesture.logic.Logic;	
	import de.davidochmann.utilities.dogesture.stack.Stack;	
	import de.davidochmann.utilities.dogesture.events.DoGestureEvent;	

	public class DoGesture extends EventDispatcher 
	{	
		private var mSurface:DisplayObject;
		private var mTimeout:uint;
		private var mMoveArray:Array;
		private var mObject:Object;
		
		private var mStack:Stack;
		private var mLogic:Logic;
		


		public function DoGesture(pSurface:DisplayObject,
								  pTimeout:uint    = 0,
								  pMoveArray:Array = null,
								  pStart:Boolean   = false,
								  pObject:Object   = null):void
		{
			setSurface(pSurface);
			setTimeout(pTimeout);
			setMoveArray(pMoveArray);
			setObject(pObject);
			
			initStack();
			initLogic();
			initEventHandler();
			initMoveArray();
						
			if(pStart) start();
		}
		
		
		
		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function start():void
		{
			mLogic.start();
		}
		
		public function stop():void
		{
			mLogic.stop();
		}
		
		public function active():Boolean
		{
			return mLogic.active();
		}

		public function complete():Boolean
		{
			return mLogic.complete();
		}

		public function add(pPositionX:int, pPositionY:int):void
		{			
			var move:Move = new Move(pPositionX, pPositionY);
			mStack.add(move);
		}


		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getSurface():DisplayObject
		{
			return mSurface;
		}

		public function getTimeout():uint
		{
			return mTimeout;
		}

		public function getMoveArray():Array
		{
			return mMoveArray;
		}
		
		public function getObject():Object
		{
			return mObject;
		}

		
		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setSurface(pSurface:DisplayObject):void
		{
			mSurface = pSurface;
		}

		public function setTimeout(pTimeout:uint):void
		{
			mTimeout = pTimeout;
		}
		
		public function setMoveArray(pMoveArray:Array):void
		{
			mMoveArray = pMoveArray;
		}
		
		public function setObject(pObject:Object):void
		{
			mObject = pObject;
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initStack():void
		{
			mStack = new Stack();
		}
		
		private function initLogic():void
		{
			mLogic = new Logic(mStack, mSurface, mTimeout, mObject);	
		}
		
		private function initEventHandler():void
		{
			mLogic.addEventListener(DoGestureEvent.MOVE_COMPLETE, logicMoveCompleteHandler);
			mLogic.addEventListener(DoGestureEvent.GESTURE_COMPLETE, logicMoveCompleteHandler);
		}
		
		private function initMoveArray():void
		{
			if(mMoveArray == null) return;
				
			var obj:Object;
			var move:Move;
				
			for(var i:uint = 0; i < mMoveArray.length; ++i)
			{
				obj  = Object(mMoveArray[i]);
				move = new Move(obj.x, obj.y);
				
				mStack.add(move);
			}			
		}
		
		private function logicMoveCompleteHandler(e:DoGestureEvent):void
		{
			dispatchEvent(e);
		}
	}
}
